﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace FantasiaPhone
{
    public delegate void ColorSelectedHandler(UserControl sender, Color c);
    public partial class CustomWheelColorPicker : UserControl
    {
        public event ColorSelectedHandler ColorSelected;

        private const int _width = 360;
        private const int _height = 360;
        private const int CX = _width / 2;
        private const int CY = _height / 2;
        private const int CZ = _depth / 2;

        SolidColorBrush _brush = new SolidColorBrush(Colors.White);
        SolidColorBrush _brushResetSample = new SolidColorBrush(Colors.White);

        int[] _pixels;
        public CustomWheelColorPicker()
        {
            InitializeComponent();
            if (!DesignerProperties.IsInDesignTool)
            {
                _pixels = Globals.BitmapColorWheel.Pixels;
            }

            InitSavedAndRecentColors();
            //markerMainLarge.Fill = _brush;
            markerMain.Fill = _brush;
            rectNewColor.Fill = _brush;
			pickerColor.Stroke = _brush;
            pathResetSample.Fill = _brushResetSample;
        }

        void InitSavedAndRecentColors()
        {
            UIElementCollection children = panelLast.Children;
            int count = children.Count;
            MouseButtonEventHandler handler = new MouseButtonEventHandler(Rectangle_MouseLeftButtonDown);
            for (int i = 1; i < count; i++)
            {
                Border border = (Border)children[i];
                border.MouseLeftButtonDown += handler;
                border.Tag = i - 1;

                Rectangle child = (Rectangle)border.Child;
                child.Fill = Globals.RecentAndSavedBrushes[i - 1];
                child.Tag = i - 1;
            }

            children = panelSaved.Children;
            count = children.Count;
            for (int i = 1; i < count; i++)
            {
                Border border = (Border)children[i];
                border.MouseLeftButtonDown += handler;
                border.Tag = Globals.RecentColorCount + i - 1;

                Rectangle child = (Rectangle)border.Child;
                child.Fill = Globals.RecentAndSavedBrushes[Globals.RecentColorCount + i - 1];
                child.Tag = Globals.RecentColorCount + i - 1;
            }
        }

        Rectangle _lastRectClicked = null;
        ScaleTransform _scale = new ScaleTransform() { CenterX = 21, CenterY = 21, ScaleX = 1.2, ScaleY = 1.2 };

        /// <summary>
        /// if >= 0, index of color to overwrite a saved color
        /// if less than 0, not in overwrite mode
        /// </summary>
        int _overwriteIndex = -1;

        const string RecentColorHint = "Tap selected color again to deselect\r\nTap saved color to overwrite with recent color.";
        const string SavedColorHint = "Tap selected color again to deselect";

        private void Rectangle_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            Border border = (Border)sender;
            Rectangle rect = (Rectangle)border.Child;
            int newColorIndex = (int)border.Tag;
            int overIndex = -1;
            if (_overwriteIndex >= 0)
            {
                panelOverwrite.Visibility = Visibility.Collapsed;
                _overwriteIndex = -1;
            }

            if (_lastRectClicked != null)
            {
                _lastRectClicked.RenderTransform = null;
                if (rect == _lastRectClicked)
                {
                    _lastRectClicked = null;
                    textHint.Visibility = Visibility.Collapsed;
                    return;
                }

                int oldColorIndex = (int)_lastRectClicked.Tag;
                if ((oldColorIndex < Globals.RecentColorCount) && (newColorIndex >= Globals.RecentColorCount)) // if previously tapped color was recent color, and new color is saved color
                {
                    // show option to overwrite (save the recent color)
                    textHint.Visibility = Visibility.Collapsed;
                    rectSaved.Fill = Globals.RecentAndSavedBrushes[newColorIndex];
                    rectOver.Fill = Globals.RecentAndSavedBrushes[oldColorIndex];
                    panelOverwrite.Visibility = Visibility.Visible;
                    overIndex = oldColorIndex;
                }
                else
                {
                    if (newColorIndex < Globals.RecentColorCount) // if newly selected color is Recent color
                    {
                        textHint.Text = RecentColorHint;
                    }
                    else
                    {
                        textHint.Text = SavedColorHint;
                    }
                    textHint.Visibility = Visibility.Visible;
                }
            }
            else
            {
                if (newColorIndex < Globals.RecentColorCount) // if newly selected color is Recent color
                {
                    textHint.Text = RecentColorHint;
                }
                else
                {
                    textHint.Text = SavedColorHint;
                }
                textHint.Visibility = Visibility.Visible;
            }

            _lastRectClicked = rect;
            _lastRectClicked.RenderTransform = _scale;

            ColorX = Globals.RecentAndSavedColors[newColorIndex].X;
            ColorY = Globals.RecentAndSavedColors[newColorIndex].Y;
            ColorZ = Globals.RecentAndSavedColors[newColorIndex].Z;
            _color = Globals.RecentAndSavedColors[newColorIndex].Color;
            ColorChanged();
            _overwriteIndex = overIndex;
        }

        bool _colorInvalid = false;
        Color _color = Colors.White;
        public Color Color
        {
            get
            {
                if (_colorInvalid)
                {
                    Color wheelColor = WheelColor;
                    if (_colorZ <= CZ)
                    {
                        float normalizedOffset = ((float)_colorZ) / CZ; // 0..1 == white...wheelColor
                        _color.R = (byte)(255 * (1 - normalizedOffset) + wheelColor.R * normalizedOffset);
                        _color.G = (byte)(255 * (1 - normalizedOffset) + wheelColor.G * normalizedOffset);
                        _color.B = (byte)(255 * (1 - normalizedOffset) + wheelColor.B * normalizedOffset);
                    }
                    else
                    {
                        float normalizedOffset = ((float)(_colorZ - CZ)) / CZ; // 0..1 == wheelColor...black
                        normalizedOffset = 1 - normalizedOffset;
                        _color.R = (byte)(wheelColor.R * normalizedOffset);
                        _color.G = (byte)(wheelColor.G * normalizedOffset);
                        _color.B = (byte)(wheelColor.B * normalizedOffset);
                    }
                    _colorInvalid = false;
                }

                return _color;
            }

            //private set
            //{
            //    _color = value;
            //    ColorChanged();
            //}
        }

        bool _wheelColorInvalid = false;
        Color _wheelColor = Colors.White;
        private Color WheelColor
        {
            get
            {
                if (_wheelColorInvalid)
                {
                    uint value = unchecked((uint)_pixels[_colorY * _width + _colorX]);
                    _wheelColor.A = (byte)(value >> 24);
                    _wheelColor.R = (byte)((value >> 16) & 0xFF);
                    _wheelColor.G = (byte)((value >> 8) & 0xFF);
                    _wheelColor.B = (byte)(value & 0xFF);
                    _wheelColorInvalid = false;
                }

                return _wheelColor;
            }
        }

        int _colorX = CX;
        public int ColorX
        {
            get
            {
                return _colorX;
            }
            set
            {
                _colorX = value;
                if (_colorX < 0) _colorX = 0;
                if (_colorX >= _width) _colorX = _width - 1;

                Canvas.SetLeft(markerMain, _colorX - 16);
                Canvas.SetLeft(markerShadow, _colorX - 15);
				Canvas.SetLeft(picker, _colorX);
				//Canvas.SetLeft(markerMainLarge, _colorX - 81);
                //Canvas.SetLeft(markerShadowLarge, _colorX - 80);
                _wheelColorInvalid = true;
                _colorInvalid = true;

                UpdateViewColors();
            }
        }

        int _colorY = CY;
        public int ColorY
        {
            get
            {
                return _colorY;
            }
            set
            {
                _colorY = value;
                if (_colorY < 0) _colorY = 0;
                if (_colorY >= _height) _colorY = _height - 1;

                Canvas.SetTop(markerMain, _colorY - 16);
				Canvas.SetTop(markerShadow, _colorY - 16);
				Canvas.SetTop(picker, _colorY);
                //Canvas.SetTop(markerMainLarge, _colorY - 81);
                //Canvas.SetTop(markerShadowLarge, _colorY - 80);
                _wheelColorInvalid = true;
                _colorInvalid = true;

                UpdateViewColors();
            }
        }

        const int _depth = 256;
        int _colorZ = CZ;
        public int ColorZ
        {
            get
            {
                return _colorZ;
            }
            set
            {
                _colorZ = value;
                if (_colorZ < 0) _colorZ = 0;
                if (_colorZ >= _depth) _colorZ = _depth - 1;

                Canvas.SetTop(panelMarker, _colorZ);
                _colorInvalid = true;

                UpdateViewColors();
            }
        }

        private void UpdateColor(Point pos)
        {
            ColorX = (int)pos.X;
            ColorY = (int)pos.Y;
            ColorChanged();
        }

        private void UpdateColor(int z)
        {
            ColorZ = z;
            ColorChanged();
        }

        private void ColorChanged()
        {
            if (_lastRectClicked != null)
            {
                int index = (int)_lastRectClicked.Tag;
                Globals.RecentAndSavedColors[index].Color = _color;
                Globals.RecentAndSavedColors[index].X = _colorX;
                Globals.RecentAndSavedColors[index].Y = _colorY;
                Globals.RecentAndSavedColors[index].Z = _colorZ;
                Globals.RecentAndSavedBrushes[index].Color = _color;
                if (_overwriteIndex >= 0)
                {
                    panelOverwrite.Visibility = Visibility.Collapsed;
                    _overwriteIndex = -1;
                }
            }

            if (ColorSelected != null)
            {
                ColorSelected(this, Color);
            }
        }
        private void UpdateViewColors()
        {
            sampleColor.Color = WheelColor;
            _brushResetSample.Color = WheelColor;
            _brush.Color = Color;
        }

        bool _isLeftButtonDownInWheel = false;
        private void Canvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (!_isLeftButtonDownInWheel) return;
            UpdateColor(e.GetPosition(panelWheel));
        }

        private void Canvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _isLeftButtonDownInWheel = true;
            markerMain.Visibility = Visibility.Collapsed;
            markerShadow.Visibility = Visibility.Collapsed;
			panelPicker.Visibility = Visibility.Visible;

            //markerMainLarge.Visibility = Visibility.Visible;
            //markerShadowLarge.Visibility = Visibility.Visible;
            panelTouchTarget.CaptureMouse();
            UpdateColor(e.GetPosition(panelWheel));
        }

        private void Canvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isLeftButtonDownInWheel = false;
            markerMain.Visibility = Visibility.Visible;
            markerShadow.Visibility = Visibility.Visible;
			panelPicker.Visibility = Visibility.Collapsed;
            //markerMainLarge.Visibility = Visibility.Collapsed;
            //markerShadowLarge.Visibility = Visibility.Collapsed;
            panelTouchTarget.ReleaseMouseCapture();
        }

        bool _isLeftButtonDownInFineColor = false;
        private void panelFineColor_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _isLeftButtonDownInFineColor = true;
            panelFineColor.CaptureMouse();
            UpdateColor((int)e.GetPosition(panelFineColor).Y);
        }

        private void panelFineColor_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isLeftButtonDownInFineColor = false;
            panelFineColor.ReleaseMouseCapture();
        }

        private void panelFineColor_MouseMove(object sender, MouseEventArgs e)
        {
            if (!_isLeftButtonDownInFineColor) return;
            UpdateColor((int)e.GetPosition(panelFineColor).Y);
        }

        private void rectResetSample_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            UpdateColor(CZ);
        }

        private void rectNewColor_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            UpdateColor(CZ);
        }

        private void panelOverwrite_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            panelOverwrite.Visibility = Visibility.Collapsed;
            if (_overwriteIndex < 0) return;

            if (_lastRectClicked != null)
            {
                ColorX = Globals.RecentAndSavedColors[_overwriteIndex].X;
                ColorY = Globals.RecentAndSavedColors[_overwriteIndex].Y;
                ColorZ = Globals.RecentAndSavedColors[_overwriteIndex].Z;
                _color = Globals.RecentAndSavedColors[_overwriteIndex].Color;
                _overwriteIndex = -1;
                ColorChanged();
                return;
            }

            _overwriteIndex = -1;
        }

    }
}
