﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Shapes;
using System.Windows.Media.Animation;

namespace FantasiaPhone
{
    [TemplatePart(Name = "rectFore", Type = typeof(Rectangle))]
    public class EasingProgressBar : Control
    {
        public static readonly DependencyProperty PercentProperty = DependencyProperty.Register("Percent", typeof(double), typeof(EasingProgressBar), new PropertyMetadata(new PropertyChangedCallback(OnPercentChanged)));

        double _percent = 0;

        /// <summary>
        /// Percent complete in the range 0..100
        /// </summary>
        public double Percent
        {
            get
            {
                return (double)this.GetValue(PercentProperty);
            }
            set
            {
                _percent = value;
                base.SetValue(PercentProperty, value);
            }
        }

        Rectangle rectFore = null;
        public EasingProgressBar()
        {
            this.DefaultStyleKey = typeof(EasingProgressBar);
        }

        public override void OnApplyTemplate()
        {
            rectFore = GetTemplateChild("rectFore") as Rectangle;
            this.SizeChanged += new SizeChangedEventHandler(EasingProgressBar_SizeChanged);
            base.OnApplyTemplate();
        }

        void EasingProgressBar_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            UpdatePercent();
        }

        private static void OnPercentChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            EasingProgressBar bar = d as EasingProgressBar;
            if (bar == null) return;
            bar.UpdatePercent();
        }

        CubicEase _easeOut = new CubicEase() { EasingMode = EasingMode.EaseOut };

        private void UpdatePercent()
        {
            if (rectFore == null) return;
            double width = this.ActualWidth;
            double normalized = _easeOut.Ease(_percent / 100);
            if ((width > 0) && (!double.IsInfinity(width))) 
            {
                rectFore.Width = normalized * width;
            }
            else {
                rectFore.Width = 0;
            }
        }
    }
}
