﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace FantasiaPhone {
    public partial class EditableSliderPhone : UserControl {

        private bool _editOnClickSetting = false;
        public delegate void ValueChangedHandler(EditableSliderPhone sender, double newValue);

        public event ValueChangedHandler ValueChanged;

        bool _isInitialized = false;
        public EditableSliderPhone()
        {
            InitializeComponent();
            _isInitialized = true;

            this.SizeChanged += new SizeChangedEventHandler(EditableSlider_SizeChanged);
        }

        void EditableSlider_SizeChanged(object sender, SizeChangedEventArgs e) {
            if (!_isInitialized) return;
            UpdateView();
        }

        /// <summary>
        /// True if dragging via mouse, false otherwise
        /// </summary>
        private bool _isDragging = false;

        private double _minimum = 0;
        private double _maximum = 10;
        private double _value = 3;
        private string _displayFormat = "0.00";
        private bool _usingCustomDisplayFormat = false;

        public double Minimum {
            get { return _minimum; }
            set { _minimum = value; UpdateView(); }
        }

        public double Maximum {
            get { return _maximum; }
            set { _maximum = value; UpdateView(); }
        }

        public string Caption
        {
            get { return textCaption.Text; }
            set { textCaption.Text = value; }
        }

        public double Value {
            get { return _value; }
            set { 
                _value = value; 
                UpdateView(); 
                UpdateText();
                if (ValueChanged != null) ValueChanged(this, _value);
            }
        }

        public string DisplayFormat {
            get { return _displayFormat; }
            set {
                _usingCustomDisplayFormat = true;
                if (_displayFormat == value) return;
                _displayFormat = value; 
                UpdateText();
            }
        }

        private void rectBase_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
            rootGrid.CaptureMouse();
            _isDragging = true;
        }

        private void rectBase_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            ReleaseMouseCapture();
            if (_editOnClickSetting) {
                _isDragging = false;
            }
            else {
                if (_isDragging) {
                    UpdatePercent(e.GetPosition(rectBase).X);
                }
                _isDragging = false;
            }
        }

        private void rectBase_MouseMove(object sender, MouseEventArgs e) {
            if (_isDragging) {
                UpdatePercent(e.GetPosition(rectBase).X);
            }
        }

        private void UpdatePercent(double xPos) {
            if (xPos < 0) xPos = 0;
            if (xPos > rectBase.ActualWidth) xPos = rectBase.ActualWidth;
            rectPercent.Width = xPos;

            double valuePercent = xPos / rectBase.ActualWidth;
            Value = (_maximum - _minimum) * valuePercent + _minimum;
        }

        private void ChangeDisplayFormatIfNeeded(string newFormat) {
            if (_usingCustomDisplayFormat) return;
            if (_displayFormat == newFormat) return;
            _displayFormat = newFormat;
            UpdateText();
        }

        private void UpdateView() {
            double valuePercent = (_value - _minimum) / (_maximum - _minimum);
            if (valuePercent < 0) valuePercent = 0; // sometimes when mixing float and double, this can get negative due to rounding errors

            rectPercent.Width = rectBase.ActualWidth * valuePercent;

            double range = Math.Abs(_maximum - _minimum);
            if (range >= 40) {
                ChangeDisplayFormatIfNeeded("0");
            }
            else 
            if (range >= 10) {
                ChangeDisplayFormatIfNeeded("0.0");
            }
            if (range >= 0.5) {
                ChangeDisplayFormatIfNeeded("0.00");
            }
            else {
                ChangeDisplayFormatIfNeeded("0.0000");
            }
        }

        private void UpdateText() {
            textValue.Text = _value.ToString(_displayFormat);
        }
    }
}
