﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Fantasia.Imaging.Silverlight;

namespace FantasiaPhone
{
    public static class Globals
    {


        public const int RecentColorCount = 6;
        public static CompositeColor[] RecentAndSavedColors = new CompositeColor[RecentColorCount * 2];
        public static SolidColorBrush[] RecentAndSavedBrushes = new SolidColorBrush[RecentColorCount * 2];

        public static void AddRecentColor(CompositeColor color)
        {
            // search to see if color is already existing
            int indexToMoveToTop = -1;
            for (int i = 0; i < RecentColorCount; i++)
            {
                if (RecentAndSavedColors[i].Color == color.Color)
                {
                    indexToMoveToTop = i;
                    break;
                }
            }

            CompositeColor c;
            if (indexToMoveToTop >= 0) // if existing
            {
                // move to top
                c = RecentAndSavedColors[indexToMoveToTop];
                for (int i = indexToMoveToTop + 1; i < RecentColorCount; i++)
                {
                    RecentAndSavedColors[i - 1] = RecentAndSavedColors[i];
                    RecentAndSavedBrushes[i - 1].Color = RecentAndSavedBrushes[i].Color;
                }
                RecentAndSavedColors[RecentColorCount - 1] = c;
                RecentAndSavedBrushes[RecentColorCount - 1].Color = color.Color;
                return;
            }

            // remove last color and add new one
            c = RecentAndSavedColors[0];
            for (int i = 1; i < RecentColorCount; i++)
            {
                RecentAndSavedColors[i - 1] = RecentAndSavedColors[i];
                RecentAndSavedBrushes[i - 1].Color = RecentAndSavedBrushes[i].Color;
            }
            RecentAndSavedColors[RecentColorCount - 1] = c;
            c.Color = color.Color;
            c.X = color.X;
            c.Y = color.Y;
            c.Z = color.Z;
            RecentAndSavedBrushes[RecentColorCount - 1].Color = color.Color;
        }

        static Globals()
        {
            for (int i = 0; i < RecentAndSavedColors.Length; i++)
            {
                RecentAndSavedColors[i] = new CompositeColor(Colors.Gray, 180, 180, 128);
                RecentAndSavedBrushes[i] = new SolidColorBrush(Colors.Gray);
            }
        }

        private static WriteableBitmap _bitmapColorWheel = null;
        public static WriteableBitmap BitmapColorWheel
        {
            get
            {
                if (_bitmapColorWheel == null)
                {
                    Stream stream = Application.GetResourceStream(new Uri("NokolaPhonePack1;component/Images/wheel.png", UriKind.Relative)).Stream;
                    _bitmapColorWheel = new WriteableBitmap(1, 1); // OK
                    _bitmapColorWheel.SetSource(stream);
                }

                return _bitmapColorWheel;
            }
        }

        public static class Bitmaps
        {
            /// <summary>
            /// Line color picker
            /// </summary>
            public static WriteableBitmap LineColorPicker = GetBitmap("/Images/linePicker.png");
        }

        private static WriteableBitmap GetBitmap(string url)
        {
            WriteableBitmap result = new WriteableBitmap(1, 1); // OK
            result.SetSource(Application.GetResourceStream(new Uri("NokolaPhonePack1;component" + url, UriKind.Relative)).Stream);
            return result;
        }
    }
}
