﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Fantasia.Imaging.Silverlight.Utilities;

namespace FantasiaPhone
{
    public partial class SmallColorPicker : UserControl
    {
        public event ColorSelectedHandler ColorSelected;
		public delegate void PickingColorHandler(bool isPickingColor);
		public event PickingColorHandler PickingColor;

        public SmallColorPicker()
        {
            InitializeComponent();
            
            _linePickerPixels = Globals.Bitmaps.LineColorPicker.Pixels;
        
            _rectColorFill = new SolidColorBrush(_color);
            rectColor.Fill = _rectColorFill;

            _brushPicker = new SolidColorBrush();
            pickerColor.Stroke = _brushPicker;
        }

        int[] _linePickerPixels;
        SolidColorBrush _rectColorFill;
        SolidColorBrush _brushPicker;

        public string Caption
        {
            get { return textCaption.Text; }
            set { textCaption.Text = value; }
        }

        private Color _color = Colors.White;
        public Color Color
        {
            get
            {
                return _color;
            }
            set
            {
                _color = value;
                _rectColorFill.Color = _color;

                int x, y;
                ColorHelper.GetLinePickerCoordinatesFromColor(_color.R, _color.G, _color.B, out x, out y);
                Canvas.SetLeft(panelMarker, x - 16);
                Canvas.SetTop(panelMarker, y - 16);
                ColorChanged();
            }
        }

        private bool _isCompact = false;
        public bool IsCompact
        {
            get
            {
                return _isCompact;
            }
            set
            {
                if (_isCompact == value) return;
                _isCompact = value;
                if (_isCompact)
                {
                    Grid.SetRow(rectColor, 1);
                    Grid.SetColumn(rectColor, 0);
                    rectColor.Width = 24;
                    rectColor.Height = 24;
                    rectColor.Margin = new Thickness(0, 0, 8, 0);

                    textCaption.Foreground = new SolidColorBrush(Colors.White);
                    textCaption.Margin = new Thickness(4);
                    textCaption.VerticalAlignment = VerticalAlignment.Bottom;
                    Grid.SetRow(textCaption, 1);


                }
            }
        }

        bool _isLeftButtonDownInWheel = false;
        private void Canvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (!_isLeftButtonDownInWheel) return;
            UpdateColor(e.GetPosition(panelData));
        }

        private void Canvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _isLeftButtonDownInWheel = true;
            panelMarker.Visibility = Visibility.Collapsed;
            panelPicker.Visibility = Visibility.Visible;
            panelTouchTarget.CaptureMouse();
            UpdateColor(e.GetPosition(panelData));
			if (PickingColor != null) PickingColor(true);
        }

        private void Canvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isLeftButtonDownInWheel = false;
            panelMarker.Visibility = Visibility.Visible;
            panelPicker.Visibility = Visibility.Collapsed;
            panelTouchTarget.ReleaseMouseCapture();
			if (PickingColor != null) PickingColor(false);
        }

        private void UpdateColor(Point pos)
        {
            int x = (int)pos.X;
            int y = (int)pos.Y;

            if (x < 0) x = 0;
            if (y < 0) y = 0;
            if (x >= ColorHelper.QUICK_PICKER_WIDTH) x = ColorHelper.QUICK_PICKER_WIDTH - 1;
            if (y >= ColorHelper.QUICK_PICKER_HEIGHT) y = ColorHelper.QUICK_PICKER_HEIGHT - 1;

            int color = _linePickerPixels[y * ColorHelper.QUICK_PICKER_WIDTH + x];
            Color colorObject = Color.FromArgb(255, (byte)(color >> 16), (byte)(color >> 8), (byte)color);
            _color = colorObject;
            _rectColorFill.Color = _color;
            _brushPicker.Color = _color;

            Canvas.SetLeft(panelMarker, x - 16);
            Canvas.SetTop(panelMarker, y - 16);

            Canvas.SetLeft(panelPicker, x);
            Canvas.SetTop(panelPicker, y);

            ColorChanged();
        }

        private void ColorChanged()
        {
            if (ColorSelected != null)
            {
                ColorSelected(this, Color);
            }
        }
    }
}
